@@ -1,13 +1,14 @@
-from pathlib import Path
+import platform
 import shutil
 import subprocess
-import pytest
-import platform
 import sys
+from pathlib import Path
+
+import pytest
 import tomli
 import tomli_w
 
-from .common import CURRENT_PLATFORM, verify_cli_command, ExitCode
+from .common import CONDA_FORGE_CHANNEL, CURRENT_PLATFORM, ExitCode, verify_cli_command
 
 
 @pytest.mark.extra_slow
@@ -42,7 +43,7 @@ def test_python_mismatch(pixi: Path, tmp_pixi_workspace: Path) -> None:
     )
 
 
-@pytest.mark.slow
+@pytest.mark.extra_slow
 def test_prefix_only_created_when_sdist(
     pixi: Path, tmp_pixi_workspace: Path, tmp_path: Path
 ) -> None:
@@ -340,6 +341,7 @@ def test_build_git_source_deps(
     )
 
 
+@pytest.mark.slow
 def test_installation_pypi_conda_mismatch(
     pixi: Path, tmp_pixi_workspace: Path, test_data: Path, pixi_tomls: Path
 ) -> None:
@@ -413,8 +415,9 @@ def test_installation_pypi_conda_mismatch(
     assert (site_packages / "foobar" / "b.py").exists(), "b.py does not exist"
 
 
+@pytest.mark.slow
 def test_pypi_url_fragment_in_project_deps(tmp_pixi_workspace: Path, pixi: Path) -> None:
-    pyproject_content = """
+    pyproject_content = f"""
 [project]
 version = "0.1.0"
 name = "test"
@@ -428,11 +431,11 @@ requires = ["hatchling"]
 build-backend = "hatchling.build"
 
 [tool.pixi.workspace]
-platforms = ["linux-64", "osx-arm64", "win-64"]
-channels = ["https://prefix.dev/conda-forge"]
+platforms = ["{CURRENT_PLATFORM}"]
+channels = ["{CONDA_FORGE_CHANNEL}"]
 
 [tool.pixi.pypi-dependencies]
-test = { path = ".", editable = true }
+test = {{ path = ".", editable = true }}
 
 [tool.hatch.metadata]
 allow-direct-references = true
@@ -476,7 +479,7 @@ def test_help_warning_when_platform_not_supported(pixi: Path, tmp_pixi_workspace
     )
 
 
-@pytest.mark.slow
+@pytest.mark.extra_slow
 def test_issue_4123_cache_prevents_editable_install(
     pixi: Path, tmp_pixi_workspace: Path, tmp_path: Path
 ) -> None:
@@ -516,7 +519,7 @@ build-backend = "hatchling.build"
 
     (proj1 / "pixi.toml").write_text(f"""[project]
 name = "proj1"
-channels = ["conda-forge"]
+channels = ["{CONDA_FORGE_CHANNEL}"]
 platforms = ["{CURRENT_PLATFORM}"]
 [dependencies]
 python = "3.12.*"
@@ -531,7 +534,7 @@ six = "==1.16.0"
 
     (proj2 / "pixi.toml").write_text(f"""[project]
 name = "proj2"
-channels = ["conda-forge"]
+channels = ["{CONDA_FORGE_CHANNEL}"]
 platforms = ["{CURRENT_PLATFORM}"]
 [dependencies]
 python = "3.12.*"
