@@ -3,7 +3,7 @@ import pytest
 import shutil
 import sys
 
-from .common import ExitCode, verify_cli_command
+from .common import verify_cli_command
 
 
 @pytest.mark.slow
@@ -15,7 +15,6 @@ def test_install_with_uv_sources(
     shutil.copytree(test_data / "uv-sources-non-root", tmp_pixi_workspace, dirs_exist_ok=True)
     verify_cli_command(
         [pixi, "install", "--manifest-path", tmp_pixi_workspace],
-        expected_exit_code=ExitCode.SUCCESS,
     )
     # Check if dist-info is available for local-library2
     #
